<?php

namespace Islhd\Site\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Islhd\Site\Http\Requests\CenterRequest;
use Islhd\Site\Interfaces\CenterRepositoryInterface;
use Islhd\Site\Models\Center;

/**
 * Resource controller class for center.
 */
class CenterResourceController extends BaseController
{

    /**
     * Initialize center resource controller.
     *
     * @param type CenterRepositoryInterface $center
     *
     * @return null
     */
    public function __construct(CenterRepositoryInterface $center)
    {
        parent::__construct();
        $this->repository = $center;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Islhd\Site\Repositories\Criteria\CenterResourceCriteria::class);
    }

    /**
     * Display a list of center.
     *
     * @return Response
     */
    public function index(CenterRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Islhd\Site\Repositories\Presenter\CenterPresenter::class)
                ->$function();
        }

        $centers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('site::center.names'))
            ->view('site::center.index', true)
            ->data(compact('centers', 'view'))
            ->output();
    }

    /**
     * Display center.
     *
     * @param Request $request
     * @param Model   $center
     *
     * @return Response
     */
    public function show(CenterRequest $request, Center $center)
    {

        if ($center->exists) {
            $view = 'site::center.show';
        } else {
            $view = 'site::center.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('site::center.name'))
            ->data(compact('center'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new center.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CenterRequest $request)
    {

        $center = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('site::center.name')) 
            ->view('site::center.create', true) 
            ->data(compact('center'))
            ->output();
    }

    /**
     * Create new center.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CenterRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $center                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('site::center.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('site/center/' . $center->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/site/center'))
                ->redirect();
        }

    }

    /**
     * Show center for editing.
     *
     * @param Request $request
     * @param Model   $center
     *
     * @return Response
     */
    public function edit(CenterRequest $request, Center $center)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('site::center.name'))
            ->view('site::center.edit', true)
            ->data(compact('center'))
            ->output();
    }

    /**
     * Update the center.
     *
     * @param Request $request
     * @param Model   $center
     *
     * @return Response
     */
    public function update(CenterRequest $request, Center $center)
    {
        try {
            $attributes = $request->all();

            $center->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('site::center.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('site/center/' . $center->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('site/center/' . $center->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the center.
     *
     * @param Model   $center
     *
     * @return Response
     */
    public function destroy(CenterRequest $request, Center $center)
    {
        try {

            $center->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('site::center.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('site/center/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('site/center/' . $center->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple center.
     *
     * @param Model   $center
     *
     * @return Response
     */
    public function delete(CenterRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('site::center.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('site/center'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/site/center'))
                ->redirect();
        }

    }

    /**
     * Restore deleted centers.
     *
     * @param Model   $center
     *
     * @return Response
     */
    public function restore(CenterRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('site::center.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/site/center'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/site/center/'))
                ->redirect();
        }

    }

}
