<?php

namespace Islhd\Site\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Islhd\Site\Http\Requests\DistrictRequest;
use Islhd\Site\Interfaces\DistrictRepositoryInterface;
use Islhd\Site\Models\District;

/**
 * Resource controller class for district.
 */
class DistrictResourceController extends BaseController
{

    /**
     * Initialize district resource controller.
     *
     * @param type DistrictRepositoryInterface $district
     *
     * @return null
     */
    public function __construct(DistrictRepositoryInterface $district)
    {
        parent::__construct();
        $this->repository = $district;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Islhd\Site\Repositories\Criteria\DistrictResourceCriteria::class);
    }

    /**
     * Display a list of district.
     *
     * @return Response
     */
    public function index(DistrictRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Islhd\Site\Repositories\Presenter\DistrictPresenter::class)
                ->$function();
        }

        $districts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('site::district.names'))
            ->view('site::district.index', true)
            ->data(compact('districts', 'view'))
            ->output();
    }

    /**
     * Display district.
     *
     * @param Request $request
     * @param Model   $district
     *
     * @return Response
     */
    public function show(DistrictRequest $request, District $district)
    {

        if ($district->exists) {
            $view = 'site::district.show';
        } else {
            $view = 'site::district.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('site::district.name'))
            ->data(compact('district'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new district.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DistrictRequest $request)
    {

        $district = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('site::district.name')) 
            ->view('site::district.create', true) 
            ->data(compact('district'))
            ->output();
    }

    /**
     * Create new district.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DistrictRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $district                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('site::district.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('site/district/' . $district->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/site/district'))
                ->redirect();
        }

    }

    /**
     * Show district for editing.
     *
     * @param Request $request
     * @param Model   $district
     *
     * @return Response
     */
    public function edit(DistrictRequest $request, District $district)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('site::district.name'))
            ->view('site::district.edit', true)
            ->data(compact('district'))
            ->output();
    }

    /**
     * Update the district.
     *
     * @param Request $request
     * @param Model   $district
     *
     * @return Response
     */
    public function update(DistrictRequest $request, District $district)
    {
        try {
            $attributes = $request->all();

            $district->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('site::district.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('site/district/' . $district->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('site/district/' . $district->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the district.
     *
     * @param Model   $district
     *
     * @return Response
     */
    public function destroy(DistrictRequest $request, District $district)
    {
        try {

            $district->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('site::district.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('site/district/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('site/district/' . $district->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple district.
     *
     * @param Model   $district
     *
     * @return Response
     */
    public function delete(DistrictRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('site::district.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('site/district'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/site/district'))
                ->redirect();
        }

    }

    /**
     * Restore deleted districts.
     *
     * @param Model   $district
     *
     * @return Response
     */
    public function restore(DistrictRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('site::district.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/site/district'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/site/district/'))
                ->redirect();
        }

    }

}
